==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/engine/PersistentIdManager.java <==
/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jul 14, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.engine;

import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.apache.ibatis.session.SqlSession;
import org.apache.log4j.Logger;

import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.db.DB;
import pl.edu.icm.unicore.uvos.db.DBEntitiesCache;
import pl.edu.icm.unicore.uvos.model.EntityBean;

/**
 * Controls and generates persistent ids for entities (conformant to SAML 2.0 spec).
 * @author golbi
 */
public class PersistentIdManager
{
	private static final Logger log = Logger.getLogger(PersistentIdManager.class);
	private static final String PREFIX = "UVOSEntityId:";
	private Set<String> used = new HashSet<String>();
	private Random randomEngine = new Random();
	private DBEntitiesCache entitiesCache;
	private DB db;
	
	public PersistentIdManager(DBEntitiesCache entitiesCache, DB db) throws InternalException
	{
		this.entitiesCache = entitiesCache;
		this.db = db;
		readIds();
	}
	
	/**
	 * Generates a new random id and mark it as used.
	 * @return
	 */
	public synchronized String getNewId()
	{
		long rand = randomEngine.nextLong();
		if (rand < 0) 
			rand = -rand;
		String id = PREFIX + rand;
		while (used.contains(id))
			id = PREFIX + ++rand;
		used.add(id);
		return id;
	}
	
	private synchronized void readIds() throws InternalException
	{
		SqlSession session = db.getSqlSession();
		List<EntityBean> all;
		try
		{
			all = entitiesCache.getAllEntities(session);
		} catch (InternalException e)
		{
			log.error("Can't load list of existing persistent entity ids.");
			throw e;
		} finally
		{
			db.releaseSqlSession(session);
		}
		for (EntityBean eb: all)
			used.add(eb.getPersistentId());
	}
	
	/**
	 * Warning! This method shouldn't be normally used. It is intended to be 
	 * used in case of DB upgreade from the version without persistent ids.
	 * <p>
	 * Updates a list of persistent ids with a freshly generated ones.
	 * <p>
	 * Warning - after calling this method this object is no longer usable.
	 * You have to get another instance.
	 *  
	 * @throws InternalException 
	 *
	 */
	public synchronized void regenerateAll() throws InternalException
	{
		log.info("RECREATING ALL PERSISTENT ENTITY IDs.");
		System.out.println("RECREATING ALL PERSISTENT ENTITY IDs.");
		SqlSession session = db.getSqlSession();
		List<EntityBean> all;
		try
		{
			try
			{
				all = entitiesCache.getAllEntities(session);
			} catch (InternalException e)
			{
				log.error("Can't load list of existing persistent entity ids.");
				throw e;
			}
			used.clear();
			for (EntityBean eb: all)
			{
				String id = getNewId();
				entitiesCache.updateEntityPersistentId(eb.getId(), id, session);
			}
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
		log.info("ALL PERSISTENT ENTITY IDs WERE UPDATED.");
		System.out.println("ALL PERSISTENT ENTITY IDs WERE UPDATED.");
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/engine/PersistentIdManager.java <==
/*
 * Copyright (c) 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jul 14, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.engine;

import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import org.apache.ibatis.session.SqlSession;
import org.apache.log4j.Logger;

import eu.unicore.util.Log;

import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.db.DB;
import pl.edu.icm.unicore.uvos.db.DBEntitiesCache;
import pl.edu.icm.unicore.uvos.model.EntityBean;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * Controls and generates persistent ids for entities (conformant to SAML 2.0 spec).
 * @author golbi
 */
public class PersistentIdManager
{
	private static final Logger log = Log.getLogger(UVOSServerProperties.LOG_PFX, PersistentIdManager.class);
	private static final String PREFIX = "UVOSEntityId:";
	private Set<String> used = new HashSet<String>();
	private Random randomEngine = new Random();
	private DBEntitiesCache entitiesCache;
	private DB db;
	
	public PersistentIdManager(DBEntitiesCache entitiesCache, DB db) throws InternalException
	{
		this.entitiesCache = entitiesCache;
		this.db = db;
		readIds();
	}
	
	/**
	 * Generates a new random id and mark it as used.
	 * @return
	 */
	public synchronized String getNewId()
	{
		long rand = randomEngine.nextLong();
		if (rand < 0) 
			rand = -rand;
		String id = PREFIX + rand;
		while (used.contains(id))
			id = PREFIX + ++rand;
		used.add(id);
		return id;
	}
	
	private synchronized void readIds() throws InternalException
	{
		SqlSession session = db.getSqlSession();
		List<EntityBean> all;
		try
		{
			all = entitiesCache.getAllEntities(session);
		} catch (InternalException e)
		{
			log.error("Can't load list of existing persistent entity ids.");
			throw e;
		} finally
		{
			db.releaseSqlSession(session);
		}
		for (EntityBean eb: all)
			used.add(eb.getPersistentId());
	}
	
	/**
	 * Warning! This method shouldn't be normally used. It is intended to be 
	 * used in case of DB upgreade from the version without persistent ids.
	 * <p>
	 * Updates a list of persistent ids with a freshly generated ones.
	 * <p>
	 * Warning - after calling this method this object is no longer usable.
	 * You have to get another instance.
	 *  
	 * @throws InternalException 
	 *
	 */
	public synchronized void regenerateAll() throws InternalException
	{
		log.info("RECREATING ALL PERSISTENT ENTITY IDs.");
		System.out.println("RECREATING ALL PERSISTENT ENTITY IDs.");
		SqlSession session = db.getSqlSession();
		List<EntityBean> all;
		try
		{
			try
			{
				all = entitiesCache.getAllEntities(session);
			} catch (InternalException e)
			{
				log.error("Can't load list of existing persistent entity ids.");
				throw e;
			}
			used.clear();
			for (EntityBean eb: all)
			{
				String id = getNewId();
				entitiesCache.updateEntityPersistentId(eb.getId(), id, session);
			}
			session.commit();
		} finally
		{
			db.releaseSqlSession(session);
		}
		log.info("ALL PERSISTENT ENTITY IDs WERE UPDATED.");
		System.out.println("ALL PERSISTENT ENTITY IDs WERE UPDATED.");
	}
}
